@@ -1,3 +1,4 @@
+"""Abinit calculator interface."""
 # Copyright (C) 2014 Atsushi Togo
 # All rights reserved.
 #
@@ -33,65 +34,92 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 import sys
+
 import numpy as np
 
-from phonopy.file_IO import collect_forces, get_drift_forces
-from phonopy.interface.vasp import get_scaled_positions_lines
-from phonopy.units import Bohr
 from phonopy.cui.settings import fracval
-from phonopy.structure.atoms import Atoms
+from phonopy.file_IO import collect_forces
+from phonopy.interface.vasp import (
+    check_forces,
+    get_drift_forces,
+    get_scaled_positions_lines,
+)
+from phonopy.structure.atoms import PhonopyAtoms as Atoms
+from phonopy.units import Bohr
 
-def parse_set_of_forces(num_atoms, forces_filenames):
-    hook = 'cartesian forces (eV/Angstrom)'
+
+def parse_set_of_forces(num_atoms, forces_filenames, verbose=True):
+    """Parse forces from output files."""
+    hook = "cartesian forces (eV/Angstrom)"
+    is_parsed = True
     force_sets = []
-    for filename in forces_filenames:
+    for i, filename in enumerate(forces_filenames):
+        if verbose:
+            sys.stdout.write("%d. " % (i + 1))
+
         f = open(filename)
         abinit_forces = collect_forces(f, num_atoms, hook, [1, 2, 3])
-        if not abinit_forces:
-            return []
+        if check_forces(abinit_forces, num_atoms, filename, verbose=verbose):
+            drift_force = get_drift_forces(
+                abinit_forces, filename=filename, verbose=verbose
+            )
+            force_sets.append(np.array(abinit_forces) - drift_force)
+        else:
+            is_parsed = False
 
-        drift_force = get_drift_forces(abinit_forces)
-        force_sets.append(np.array(abinit_forces) - drift_force)
+    if is_parsed:
+        return force_sets
+    else:
+        return []
 
-    return force_sets
 
 def read_abinit(filename):
-    abinit_in = AbinitIn(open(filename).readlines())
+    """Read crystal structure."""
+    with open(filename) as f:
+        abinit_in = AbinitIn(f.readlines())
     tags = abinit_in.get_variables()
-    acell = tags['acell']
-    rprim = tags['rprim'].T
-    scalecart = tags['scalecart']
+    acell = tags["acell"]
+    rprim = tags["rprim"].T
+    scalecart = tags["scalecart"]
     lattice = rprim * acell
     if scalecart is not None:
         for i in range(3):
             lattice[i] *= scalecart[i]
 
-    if tags['xcart'] is not None:
-        pos_bohr = np.transpose(tags['xcart'])        
+    if tags["xcart"] is not None:
+        pos_bohr = np.transpose(tags["xcart"])
         positions = np.dot(np.linalg.inv(lattice), pos_bohr).T
-    elif tags['xangst'] is not None:
-        pos_bohr = np.transpose(tags['xangst']) / Bohr
+    elif tags["xangst"] is not None:
+        pos_bohr = np.transpose(tags["xangst"]) / Bohr
         positions = np.dot(np.linalg.inv(lattice), pos_bohr).T
-    elif tags['xred'] is not None:
-        positions = tags['xred']
+    elif tags["xred"] is not None:
+        positions = tags["xred"]
 
-    numbers = [tags['znucl'][x - 1] for x in tags['typat']]
+    numbers = [tags["znucl"][x - 1] for x in tags["typat"]]
+
+    return Atoms(numbers=numbers, cell=lattice.T, scaled_positions=positions)
 
-    return Atoms(numbers=numbers,
-                 cell=lattice.T,
-                 scaled_positions=positions)
 
 def write_abinit(filename, cell):
-    f = open(filename, 'w')
-    f.write(get_abinit_structure(cell))
+    """Write cell to file."""
+    with open(filename, "w") as f:
+        f.write(get_abinit_structure(cell))
+
+
+def write_supercells_with_displacements(
+    supercell, cells_with_displacements, ids, pre_filename="supercell", width=3
+):
+    """Write supercells with displacements to files."""
+    write_abinit("%s.in" % pre_filename, supercell)
+    for i, cell in zip(ids, cells_with_displacements):
+        filename = "{pre_filename}-{0:0{width}}.in".format(
+            i, pre_filename=pre_filename, width=width
+        )
+        write_abinit(filename, cell)
 
-def write_supercells_with_displacements(supercell,
-                                        cells_with_displacements):
-    write_abinit("supercell.in", supercell)
-    for i, cell in enumerate(cells_with_displacements):
-        write_abinit("supercell-%03d.in" % (i + 1), cell)
 
 def get_abinit_structure(cell):
+    """Return abinit structure in text."""
     znucl = []
     numbers = cell.get_atomic_numbers()
     for n in numbers:
@@ -115,40 +143,49 @@ def get_abinit_structure(cell):
 
     return lines
 
+
 class AbinitIn:
+    """Class to create Abinit input file."""
+
     def __init__(self, lines):
-        self._set_methods = {'acell':     self._set_acell,
-                             'natom':     self._set_natom,
-                             'ntypat':    self._set_ntypat,
-                             'rprim':     self._set_rprim,
-                             'typat':     self._set_typat,
-                             'scalecart': self._set_scalecart,
-                             'xangst':    self._set_xangst,
-                             'xcart':     self._set_xcart,
-                             'xred':      self._set_xred,
-                             'znucl':     self._set_znucl}
-        self._tags = {'acell':     None,
-                      'natom':     None,
-                      'ntypat':    None,
-                      'rprim':     None,
-                      'typat':     None,
-                      'scalecart': None,
-                      'xangst':    None,
-                      'xcart':     None,
-                      'xred':      None,
-                      'znucl':     None}
+        """Init method."""
+        self._set_methods = {
+            "acell": self._set_acell,
+            "natom": self._set_natom,
+            "ntypat": self._set_ntypat,
+            "rprim": self._set_rprim,
+            "typat": self._set_typat,
+            "scalecart": self._set_scalecart,
+            "xangst": self._set_xangst,
+            "xcart": self._set_xcart,
+            "xred": self._set_xred,
+            "znucl": self._set_znucl,
+        }
+        self._tags = {
+            "acell": None,
+            "natom": None,
+            "ntypat": None,
+            "rprim": None,
+            "typat": None,
+            "scalecart": None,
+            "xangst": None,
+            "xcart": None,
+            "xred": None,
+            "znucl": None,
+        }
 
         self._values = None
         self._collect(lines)
 
     def get_variables(self):
+        """Return tags."""
         return self._tags
 
     def _collect(self, lines):
         elements = {}
         tag = None
         for line_tmp in lines:
-            line = line_tmp.replace('!', '#').split('#')[0]
+            line = line_tmp.replace("!", "#").split("#")[0]
             for val in [x.lower() for x in line.split()]:
                 if val in self._set_methods:
                     tag = val
@@ -156,31 +193,32 @@ class AbinitIn:
                 elif tag is not None:
                     elements[tag].append(val)
 
-        for tag in ['natom', 'ntypat']:
+        for tag in ["natom", "ntypat"]:
             if tag not in elements:
                 print("%s is not found in the input file." % tag)
                 sys.exit(1)
 
-        for tag, self._values in elements.iteritems():
-            if tag == 'natom' or tag == 'ntypat':
+        for tag in elements:
+            self._values = elements[tag]
+            if tag == "natom" or tag == "ntypat":
                 self._set_methods[tag]()
 
-        for tag, self._values in elements.iteritems():
-            if tag != 'natom' and tag != 'ntypat':
+        for tag in elements:
+            self._values = elements[tag]
+            if tag != "natom" and tag != "ntypat":
                 self._set_methods[tag]()
 
-    def _get_numerical_values(self, char_string, num_type='float'):
-        vals = []
+    def _get_numerical_values(self, char_string, num_type="float"):
         m = 1
 
-        if '*' in char_string:
-            m = int(char_string.split('*')[0])
-            str_val = char_string.split('*')[1]
+        if "*" in char_string:
+            m = int(char_string.split("*")[0])
+            str_val = char_string.split("*")[1]
         else:
             m = 1
             str_val = char_string
 
-        if num_type == 'float':
+        if num_type == "float":
             a = fracval(str_val)
         else:
             a = int(str_val)
@@ -192,20 +230,20 @@ class AbinitIn:
         for val in self._values:
             if len(acell) >= 3:
                 if len(val) >= 6:
-                    if val[:6] == 'angstr':
+                    if val[:6] == "angstr":
                         for i in range(3):
                             acell[i] /= Bohr
                 break
 
             acell += self._get_numerical_values(val)
 
-        self._tags['acell'] = acell[:3]
+        self._tags["acell"] = acell[:3]
 
     def _set_natom(self):
-        self._tags['natom'] = int(self._values[0])
+        self._tags["natom"] = int(self._values[0])
 
     def _set_ntypat(self):
-        self._tags['ntypat'] = int(self._values[0])
+        self._tags["ntypat"] = int(self._values[0])
 
     def _set_rprim(self):
         rprim = []
@@ -214,7 +252,7 @@ class AbinitIn:
             if len(rprim) >= 9:
                 break
 
-        self._tags['rprim'] = np.reshape(rprim[:9], (3, 3))
+        self._tags["rprim"] = np.reshape(rprim[:9], (3, 3))
 
     def _set_scalecart(self):
         scalecart = []
@@ -223,52 +261,43 @@ class AbinitIn:
             if len(scalecart) >= 3:
                 break
 
-        self._tags['scalecart'] = np.array(scalecart[:3])
+        self._tags["scalecart"] = np.array(scalecart[:3])
 
     def _set_typat(self):
         typat = []
-        natom = self._tags['natom']
+        natom = self._tags["natom"]
         for val in self._values:
-            typat += self._get_numerical_values(val, num_type='int')
+            typat += self._get_numerical_values(val, num_type="int")
             if len(typat) >= natom:
                 break
 
-        self._tags['typat'] = typat[:natom]
+        self._tags["typat"] = typat[:natom]
 
     def _set_xangst(self):
-        self._set_x_tags('xangst')
+        self._set_x_tags("xangst")
 
     def _set_xcart(self):
-        self._set_x_tags('xcart')
+        self._set_x_tags("xcart")
 
     def _set_xred(self):
-        self._set_x_tags('xred')
+        self._set_x_tags("xred")
 
     def _set_x_tags(self, tagname):
         xtag = []
-        natom = self._tags['natom']
+        natom = self._tags["natom"]
         for val in self._values:
             xtag += self._get_numerical_values(val)
             if len(xtag) >= natom * 3:
                 break
 
-        self._tags[tagname] = np.reshape(xtag[:natom * 3], (-1, 3))
+        self._tags[tagname] = np.reshape(xtag[: natom * 3], (-1, 3))
 
     def _set_znucl(self):
         znucl = []
-        ntypat = self._tags['ntypat']
+        ntypat = self._tags["ntypat"]
         for val in self._values:
-            znucl += self._get_numerical_values(val, num_type='int')
+            znucl += self._get_numerical_values(val, num_type="int")
             if len(znucl) >= ntypat:
                 break
 
-        self._tags['znucl'] = znucl[:ntypat]
-
-if __name__ == '__main__':
-    import sys
-    from phonopy.structure.symmetry import Symmetry
-    abinit = AbinitIn(open(sys.argv[1]).readlines())
-    cell = read_abinit(sys.argv[1])
-    symmetry = Symmetry(cell)
-    print("# %s" % symmetry.get_international_table())
-    print(get_abinit_structure(cell))
+        self._tags["znucl"] = znucl[:ntypat]
